using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using PS.Practices.Provisioner.Provisioner.Util;
using PS.Practices.Provisioner.Sandbox.Util;

namespace PS.Practices.Provisioner.Sandbox.Features.PS.Provisioner.Sandbox
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("e9426ffd-e70e-4283-9a05-e2302703f865")]
    public class PSProvisionerEventReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            try
            {
                SPWeb web = properties.UserCodeSite.RootWeb;
                string listUrl = web.ServerRelativeUrl == "/"
                                         ? Constants.ProvisionerScriptsListUrl
                                         : web.ServerRelativeUrl + Constants.ProvisionerScriptsListUrl;
                SPList spList = web.GetList(listUrl);
                spList.BreakRoleInheritance(false);
                spList.Update();
            }
            catch
            {
            }
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            SPWeb web = properties.UserCodeSite.RootWeb;
            try
            {
                DeleteList(web, Constants.ProvisionerScriptsListUrl);
            }
            catch { }
            try
            {
                DeleteList(web, Constants.ProvisionerLoggerListUrl);
            }
            catch { }
        }

        private static void DeleteList(SPWeb spWeb, string listUrl)
        {
            string url = spWeb.ServerRelativeUrl == "/" ? listUrl : spWeb.ServerRelativeUrl + listUrl;
            SPList spList = spWeb.GetList(url);
            spList.Delete();
        }


        // Uncomment the method below to handle the event raised after a feature has been installed.

        //public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised before a feature is uninstalled.

        //public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        //{
        //}

        // Uncomment the method below to handle the event raised when a feature is upgrading.

        //public override void FeatureUpgrading(SPFeatureReceiverProperties properties, string upgradeActionName, System.Collections.Generic.IDictionary<string, string> parameters)
        //{
        //}
    }
}
