using System;
using System.Collections;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Web.UI.WebControls.WebParts;
using System.Xml.XPath;
using Microsoft.SharePoint;
using System.Text;

namespace PS.Practices.SiteBuilder.Features.Schema
{
    static class ParserUtil
    {
        internal static Dictionary<string, string> GetProvisionedProperties(SPWeb web)
        {
            Dictionary<string, string> retDic = new Dictionary<string, string>();

            foreach (DictionaryEntry prop in web.AllProperties)
            {
                if (prop.Key.ToString().StartsWith("ProvisionerProp_"))
                {
                    var val = prop.Key.ToString().Replace("ProvisionerProp_", "");

                    retDic.Add(val, (string)prop.Value);
                }
                    
            }

            return retDic;
        }
        
        internal static bool GetAttributeBool(XPathNavigator element, string attributeName, bool defaultValue)
        {
            bool var = defaultValue;
            string strValue = element.GetAttribute(attributeName, string.Empty);
            if (!string.IsNullOrEmpty(strValue))
            {
                bool.TryParse(strValue, out var);
            }
            return var;
        }

        internal static string GetAttribute(XPathNavigator element, string attributeName)
        {
            string strValue = element.GetAttribute(attributeName, string.Empty);
            if (!string.IsNullOrEmpty(strValue))
            {
                return strValue;
            }
            return string.Empty;
        }

        internal static WebPart UpdateWPTitle(WebPart part, SPWeb web, string setTo, Dictionary<string, string> dic)
        {
            if (part == null)
                return null;

            string val = part.Title;

            if (!string.IsNullOrEmpty(setTo))
            {
                setTo = ReplaceLiveFields(setTo, null, web);
                val = string.Format(setTo, val);
                
            }

            if (dic != null)
            {
                foreach (string key in dic.Keys)
                {
                    string value = dic[key];
                    val = val.Replace(key, value);
                }
            }

            part.Title = val;
            return part;
        }

        internal static void UpdateListTitle(SPList list, string setTo, Dictionary<string, string> dic)
        {
            string val = list.Title;

            SPWeb web = list.ParentWeb;
            if (!string.IsNullOrEmpty(setTo))
            {
                setTo = ReplaceLiveFields(setTo, list, web);
                val = string.Format(setTo, val);
            }
            
            if (dic!=null)
            {
                foreach (string key in dic.Keys)
                {
                    string value = dic[key];
                    value = ReplaceLiveFields(value, list, web);
                    val = val.Replace(key, value);
                }
            }
            
            list.Title = val;
        }


        internal static string ReplaceAllPropertyFields(string val, Dictionary<string,string> properties )
        {
            Regex r = new Regex(@"{{(?<ObjectName>\S+).(?<PropertyName>\S+)}}");
            MatchCollection mc = r.Matches(val);
            string retStr = val;

            for (int i = 0; i < mc.Count; i++)
            {
                string parsedStr = ReplacePropertiesField(mc[i].ToString(), properties);

                retStr = retStr.Replace(mc[i].ToString(), parsedStr);
            }

            return retStr;
        }

        internal static string ReplaceAllLiveFields(string val, SPWeb web)
        {
            Regex r = new Regex(@"{{(?<ObjectName>\S+).(?<PropertyName>\S+)}}");
            MatchCollection mc = r.Matches(val);
            string retStr = val;
            
            for(int i =0; i < mc.Count; i++)
            {
                string parsedStr = ReplaceLiveFields(mc[i].ToString(), web);

                retStr = retStr.Replace(mc[i].ToString(), parsedStr);
            }

            return retStr;
        }

        internal static string ReplaceLiveFields(string val, SPWeb web)
        {
            if (val == null) throw new ArgumentNullException("val");
            string v = val;
            
            if (v.Contains("{{SPWeb.Title}}") && web != null)
            {
                v = v.Replace("{{SPWeb.Title}}", web.Title);
            }

            if (v.Contains("{{SPSite.Title}}") && web != null)
            {
                v = v.Replace("{{SPSite.Title}}", web.Site.RootWeb.Title);
            }

            if (v.Contains("{{SPWeb.ServerRelativeUrl}}") && web != null)
            {
                v = v.Replace("{{SPWeb.ServerRelativeUrl}}", web.ServerRelativeUrl);
            }
            
            if (v.Contains("{{ParentWeb.Title}}") && web != null)
            {
                //ParentWeb property no longer requires Dispose() 
                SPWeb pweb = web.ParentWeb;
                v = v.Replace("{{ParentWeb.Title}}", pweb.Title);
            }

            if (v.Contains("{{SPWeb.Description}}") && web != null)
            {
                v = v.Replace("{{SPWeb.Description}}", web.Description);
            }

            if (v.Contains("{{Property"))
            {
                v = ReplacePropertiesField(v, GetProvisionedProperties(web));
            }

            if (v.Contains(@"{{SPWeb.Properties["))
            {
                v = ReplaceWebProperties(web, v);
            }

            return v;
        }

        internal static string ReplacePropertiesField(string str, Dictionary<string, string> properties)
        {
            Regex r = new Regex(@"{{Property.(?<PropertyName>\S+)}}");
            Match m = r.Match(str);
            if (m != null)
            {
                string oldString = m.Value;
                string propertyName = m.Groups["PropertyName"].Value;

                if (properties.ContainsKey(propertyName))
                {
                    string value = properties[propertyName];
                    str = str.Replace(oldString, value);
                }
            }

            return str;
        }

        internal static string ReplaceWebProperties(SPWeb web, string str)
        {
            //{{SPWeb.Properties[vti_title]}} 
            Regex r = new Regex(@"{{SPWeb.Properties\[(?<propname>\S+)\]}}");
            Match m = r.Match(str);
            if(m != null)
            {
                string oldString = m.Value;
                string propertyName = m.Groups["propname"].Value;
                if (!string.IsNullOrEmpty(propertyName))
                {
                    try
                    {
                        str = str.Replace(oldString, web.Properties[propertyName]);
                    }
                    catch (Exception)
                    {
                    }
                }
            }
            return str;
        }

        internal static string ReplaceLiveFields(string val, SPList list, SPWeb web)
        {
            if (string.IsNullOrEmpty(val)) throw new ArgumentNullException("val");
            string v = val;
            if (v.Contains("{{SPList.Title}}") && list != null)
            {
                v = v.Replace("{{SPList.Title}}", list.Title);
            }

            v = ReplaceLiveFields(v, web);

            return v;
        }

        
    }
}
