#region using

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint;
using PS.Practices.Provisioner;
using PS.Practices.SiteBuilder.Features.API;
using PS.Practices.Util;
using Microsoft.SharePoint.Utilities;
using PS.Practices.Provisioner.Provisioner.Schema.Logic;



#endregion

namespace PS.Practices.SiteBuilder.Features.Schema.DOM
{
    public abstract class WebOperation
    {

        public abstract void Accept(IWebOperationVisitor visitor);
    }

    public abstract class ListOperation
    {
        public abstract void Accept(IListOperationVisitor visitor);
    }

    public class FixupLists
        : WebOperation
    {
        private bool syncCTypeLocalFieldSettings;
        private List<List> lists = new List<List>();

        
        public bool SyncCTypeLocalFieldSettings
        {
            get { return syncCTypeLocalFieldSettings; }
            set { syncCTypeLocalFieldSettings = value; }
        }

        [XmlAttribute("SyncCTypeLocalFieldSettings")]
        public string SyncCTypeLocalFieldSettingsStr
        {
            get { return syncCTypeLocalFieldSettings.ToString(); }
            set { syncCTypeLocalFieldSettings = bool.Parse(value); }
        }

        [XmlElement("List")]
        public List<List> Lists
        {
            get { return lists; }
            set { lists = value; }
        }

        public override void Accept(IWebOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    /// <summary>
    /// Object that represents the list xml node
    /// </summary>
    public class List
    {
        private List<ListOperation> operations = new List<ListOperation>();

        public List()
        {
            Template = 100;
        }

        //! Sets the title for the provisioned list
        [XmlAttribute()]
        public string Title { get; set; }

        //! Sets the template for the list
        [XmlAttribute()]
        public int Template { get; set; }

        [XmlAttribute()]
        public string Description { get; set; }

        [XmlAttribute()]
        public bool Delete { get; set; }

        [XmlAttribute()]
        public bool EnableContentTypes { get; set; }

        [XmlElement(typeof(SetSendToLocation))]
        [XmlElement(typeof(UpdateTitle))]
        [XmlElement(typeof(ReorderCTypes))]
        [XmlElement(typeof(FixupACL))]
        [XmlElement(typeof(FixupView))]
        [XmlElement(typeof(UpdateFields))]
        [XmlElement(typeof(AddFields))]
        [XmlElement(typeof(ListItem))]
        [XmlElement(typeof(AddCTypes))]
        [XmlElement(typeof(RemoveCTypes))]
        public List<ListOperation> Operations
        {
            get { return operations; }
            set { operations = value; }
        }

        [XmlAttribute("EMailEnable")]
        public string EmailEnable { get; set; }

        [XmlAttribute("MajorVersionLimit")]
        public int MajorVersionLimit { get; set; }

        [XmlAttribute("MinorVersionLimit")]
        public int MinorVersionLimit { get; set; }

        [XmlAttribute("DraftSecurity")]
        public string DraftSecurity { get; set; }

        public void Accept(OmListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class SetSendToLocation
        : ListOperation
    {
        private string name;
        private string url;

        [XmlAttribute()]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        [XmlAttribute()]
        public string Url
        {
            get { return url; }
            set { url = value; }
        }

        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class Replace
    {
        private string match;
        private string value;

        [XmlAttribute()]
        public string Match
        {
            get { return match; }
            set { match = value; }
        }

        [XmlText()]
        public string Value
        {
            get { return value; }
            set { this.value = value; }
        }
    }

    public class UpdateTitle
        : ListOperation
    {
        public UpdateTitle()
        {
            ReplaceInfo = new List<Replace>();
        }

        //TODO: Possibly Set is excess
        [XmlAttribute()]
        public string Set { get; set; }

        [XmlElement("Replace")]
        public List<Replace> ReplaceInfo { get; set; }

        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class ContentTypeRef
    {
        private string id;
        private string name;

        [XmlAttribute()]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        [XmlAttribute()]
        public string ID
        {
            get { return id; }
            set { id = value; }
        }
    }

    public abstract class ListCTypeOperation
        : ListOperation
    {
        private List<ContentTypeRef> contentTypes = new List<ContentTypeRef>();

        [XmlElement("ContentTypeRef")]
        public List<ContentTypeRef> ContentTypes
        {
            get { return contentTypes; }
            set { contentTypes = value; }
        }
    }

    public class AddCTypes
        : ListCTypeOperation
    {
        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class RemoveCTypes
        : ListCTypeOperation
    {
        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class ReorderCTypes
        : ListOperation
    {
        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class PrincipalListRole
    {
        private string role;
        private string name;
        private string action;
        private string type;

        [XmlAttribute()]
        public string Role
        {
            get { return role; }
            set { role = value; }
        }

        [XmlAttribute()]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        [XmlAttribute()]
        public string Action
        {
            get { return action; }
            set { action = value; }
        }

        [XmlAttribute()]
        public string Type
        {
            get { return type; }
            set { type = value; }
        }

        public void Process(SPWeb web, SPList list)
        {
            name = ParserUtil.ReplaceLiveFields(name, list, web);

            DoSecurityActions(list, web);
        }

        protected void DoSecurityActions(SPSecurableObject securableObject, SPWeb web)
        {
            bool doAdd = action == "Add";
            bool doRemove = action == "Remove";

            if (!doAdd && !doRemove)
            {
                Logger.Info("Unsupported action attribute: " + action);
                return;
            }
            
            //SPPrincipal principal = null;

            PSAcl acl = new PSAcl(securableObject, web);

            switch (type)
            {
                case "SiteGroup":
                {
                    if(doAdd)
                       acl.AddSiteGroup(name, role);
                    else 
                        acl.RemoveSiteGroup(name, null);

                    break;
                }

                case "User":
                {
                    if (doAdd)
                        acl.AddUser(name, role);
                    else
                        acl.RemoveUser(name, null);

                    break;
                }

                default:
                    Logger.Info("Unsupported principal type : " + type);
                    break;
            }

            //if (principal != null)
            //{
            //    if (doAdd)
            //        AclUtil.AddPrincipal(principal, role, securableObject, web);
            //    else
            //    {
            //        AclUtil.RemovePrincipal(principal, securableObject, web);
            //    }
            //}
        }
    }

    /// <summary>
    /// Object which represents the FixupACL xml node
    /// </summary>
    public class FixupACL
        : ListOperation
    {
        public FixupACL()
        {
            Roles = new List<PrincipalListRole>();
        }

        [XmlAttribute()]
        public string RoleAssignments { get; set; }

        [XmlElement("PrincipalRole")]
        public List<PrincipalListRole> Roles { get; set; }

        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class LessField
    {
        private string name;

        [XmlAttribute()]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }
    }

    public class InjectCTypeFields
    {
        private int position;
        private List<LessField> fields = new List<LessField>();

        [XmlAttribute()]
        public int Position
        {
            get { return position; }
            set { position = value; }
        }

        [XmlElement("LessField")]
        public List<LessField> Fields
        {
            get { return fields; }
            set { fields = value; }
        }

        public void Process(SPView view, SPList list)
        {
            if (Position > view.ViewFields.Count)
                Position = view.ViewFields.Count;

            StringCollection less = new StringCollection();
            foreach (LessField lessField in fields)
            {
                less.Add(lessField.Name);
            }

            ListUtil.InjectFieldsFromCTypes(view, list.ContentTypes, Position, less);
        }
    }

    public class ViewField
    {
        private string name;

        [XmlAttribute()]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }
    }

    public class InjectViewFields
    {
        private List<ViewField> viewFields = new List<ViewField>();

        [XmlElement("ViewField")]
        public List<ViewField> ViewFields
        {
            get { return viewFields; }
            set { viewFields = value; }
        }

        public void Process(SPView view, SPList list)
        {
            view.ViewFields.DeleteAll();
            foreach (ViewField viewField in ViewFields)
            {
                view.ViewFields.Add(viewField.Name);
            }
            view.Update();
        }
    }

    public class FixupView
        : ListOperation
    {
        public FixupView()
        {
            InjectTypes = new List<InjectCTypeFields>();
            InjectFields = new List<InjectViewFields>();
        }

        [XmlAttribute()]
        public string Name { get; set; }

        [XmlElement("InjectCTypeFields")]
        public List<InjectCTypeFields> InjectTypes { get; set; }

        [XmlElement("InjectViewFields")]
        public List<InjectViewFields> InjectFields { get; set; }

        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class FieldChoices
    {
        private bool flush;
        private List<string> choices = new List<string>();

        [XmlAttribute()]
        public bool Flush
        {
            get { return flush; }
            set { flush = value; }
        }

        // TODO: check deserialization
        [XmlElement("Choice")]
        public List<string> Choices
        {
            get { return choices; }
            set { choices = value; }
        }
    }

    public class UpdateField
    {
        private string displayName;
        private string staticName;
        private string description;
        private bool dumpFields;
        private bool linkToLocalCTypes;
        private string defaultValue;
        private FieldChoices choices;

        [XmlAttribute()]
        public string DisplayName
        {
            get { return displayName; }
            set { displayName = value; }
        }

        [XmlAttribute()]
        public string StaticName
        {
            get { return staticName; }
            set { staticName = value; }
        }

        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        public bool DumpFields
        {
            get { return dumpFields; }
            set { dumpFields = value; }
        }

        [XmlAttribute()]
        public bool LinkToLocalCTypes
        {
            get { return linkToLocalCTypes; }
            set { linkToLocalCTypes = value; }
        }

        public FieldChoices Choices
        {
            get { return choices; }
            set { choices = value; }
        }

        // TODO : check deserialization
        public string Default
        {
            get { return defaultValue; }
            set { defaultValue = value; }
        }

        public void Process(SPWeb web, SPList list)
        {
            if (string.IsNullOrEmpty(DisplayName) && string.IsNullOrEmpty(StaticName))
                return;

            SPField field = null;
            try
            {
                if (!string.IsNullOrEmpty(StaticName))
                    field = list.Fields.GetFieldByInternalName(StaticName);
                else
                    field = list.Fields[DisplayName];
            }
            catch (ArgumentException)
            {
            }

            if (field == null)
            {
                Logger.Info("Could not find field " + DisplayName + ":" + StaticName+ " in list " + list.Title);
                return;
            }

            Logger.Info("Updating field " + field.StaticName + " in list " + list.Title + ". ID=" + field.Id + ", SourceID=" + field.SourceId);

            if (field.Sealed)
            {
                field.Sealed = false;
                Logger.Info("Field was sealed");
            }

            if(Description != null)
            {
                field.Description = ParserUtil.ReplaceLiveFields(Description, list, web);
                Logger.Info("Description set to: " + field.Description);
            }

            if(DumpFields)
            {
                foreach (SPField spField in list.Fields)
                {
                    Logger.Info(list.Title + ": {" + spField.Id + "} -- " + spField.StaticName + " : " + spField.Title);
                }
            }

            if(LinkToLocalCTypes)
            {
                Logger.Info("Adding field " + field.StaticName + " to all local list content types ");
                ListUtil.LinkFieldToListCTypes(field, list);
            }

            SPFieldChoice choice = field as SPFieldChoice;
            SPFieldMultiChoice mchoice = field as SPFieldMultiChoice;
            
            if (choice != null || mchoice != null)
            {
                if(Choices != null)
                {
                    StringBuilder sb = new StringBuilder();
                    StringCollection coll = choice != null ? choice.Choices : mchoice.Choices;
                    if (Choices.Flush)
                    {
                        coll.Clear();
                        sb.Append("#Flush# ");
                    }

                    foreach (string fieldChoise in Choices.Choices)
                    {
                        string s = ParserUtil.ReplaceLiveFields(fieldChoise, list, web);
                        sb.Append(s + "; ");
                        coll.Add(s);
                    }

                    Logger.Info("Choices added: " + sb);
                }
            }

            if (Default != null)
            {
                string prevDef = field.DefaultValue;
                string curDef = ParserUtil.ReplaceLiveFields(Default, list, web);
                field.DefaultValue = curDef;

                Logger.Info("Default value of '" + prevDef + "' replaced with '" + field.DefaultValue + "'");
            }

            field.Update(true);
        }
    }

    public class UpdateFields
        : ListOperation
    {
        private List<UpdateField> fields = new List<UpdateField>();

        [XmlElement("Field")]
        public List<UpdateField> Fields
        {
            get { return fields; }
            set { fields = value; }
        }

        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    // TODO : ensure correct deserialization
    public class AddField
    {
        [XmlText()]
        public string Field { get; set; }

        [XmlAttribute()]
        public bool AddToDefaultView { get; set; }
    }

    public class AddFields
        : ListOperation
    {
        public AddFields()
        {
            Fields = new List<AddField>();
        }

        [XmlElement("Field")]
        public List<AddField> Fields { get; set; }

        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }
    }

    public class ItemColumn
    {
        [XmlAttribute()]
        public string FieldName { get; set; }

        [XmlText()]
        public string Value { get; set; }
    }

    public class ListItem : ListOperation
    {
        private string type;
        private string name;
        private int id = -1;
        private FixupItemACL fixupItemACL;
        private FileStatus fileStatus;
        private List<ItemColumn> columns = new List<ItemColumn>();
        private bool syncWithPropertyBag = false;

        [XmlElement("Column")]
        public List<ItemColumn> Columns
        {
            get { return columns; }
            set { columns = value; }
        }

        [XmlElement]
        public FileStatus FileStatus
        {
            get { return fileStatus; }
            set { fileStatus = value; }
        }

        [XmlAttribute()]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        [XmlElement("FixupACL")]
        public FixupItemACL FixupItemACL
        {
            get { return fixupItemACL; }
            set { fixupItemACL = value; }
        }

        [XmlAttribute()]
        public int ID
        {
            get { return id; }
            set { id = value; }
        }

        [XmlAttribute()]
        public string Type
        {
            get { return type; }
            set { type = value; }
        }

        [XmlAttribute()]
        public bool SyncWithPropertyBag
        {
            get { return syncWithPropertyBag; }
            set { syncWithPropertyBag = value; }
        }

        public override void Accept(IListOperationVisitor visitor)
        {
            visitor.Visit(this);
        }

 
    }

    public class FileStatus
    {
 
        public enum Status
        {
            CheckIn, CheckOut, Publish
        }

        private Status status;

        [XmlAttribute]
        public Status SetStatus
        {
            get { return status; }
            set { status = value; }
        }

        public void Execute(SPWeb web, SPFile spFile)
        {
            if (spFile != null)
            {
                Logger.Info("Setting Item Status: " + spFile.Name);

                try
                {
                    switch (status)
                    {
                        case Status.CheckOut:
                            spFile.CheckOut();
                            break;
                        case Status.CheckIn:
                            spFile.CheckIn(string.Empty);
                            break;
                        case Status.Publish:
                            try
                            {
                                spFile.CheckIn(string.Empty);
                            }
                            catch (SPException)
                            {
                            }
                            spFile.Publish(string.Empty);
                            try
                            {
                                spFile.Approve(string.Empty);
                            }catch(SPException)
                            {
                            }
                            break;
                    }

                    spFile.Update();
                }
                catch (SPException)
                {
                }
            }
        }      
    }

    public class FixupItemACL
    {
        private string roleAssignments;
        private List<PrincipalItemRole> principalItemRoles;

        [XmlAttribute()]
        public string RoleAssignments
        {
            get { return roleAssignments; }
            set { roleAssignments = value; }
        }

        [XmlElement("PrincipalRole")]
        public List<PrincipalItemRole> PrincipalItemRoles
        {
            get { return principalItemRoles; }
            set { principalItemRoles = value; }
        }

        public void Execute(SPWeb web, SPList spList, SPListItem listItem)
        {
            Logger.Info("Configuring folder ACL: " + listItem.Name);

            if (!string.IsNullOrEmpty(RoleAssignments))
            {
                if (RoleAssignments.StartsWith("Break") && !listItem.HasUniqueRoleAssignments)
                {
                    Logger.Info("Breaking role inheritance: " + RoleAssignments);
                    listItem.BreakRoleInheritance(RoleAssignments == "BreakAndCopy");
                }
            }

            foreach (PrincipalItemRole principalItemRole in PrincipalItemRoles)
            {
                principalItemRole.Execute(web, spList, listItem);
            }

            web.Update();
        }
    }

    public class PrincipalItemRole :  PrincipalListRole
    {
        public void Execute(SPWeb web, SPList spList, SPListItem listItem)
        {
            this.Name = ParserUtil.ReplaceLiveFields(this.Name, spList, web);
                        
            if (listItem != null)
            {
                DoSecurityActions(listItem, web);
            }
            else
            {
                Logger.Info("Folder item cannot be null, list: " + spList.Title);
            }
        }
    }
}
