using System;
using System.Collections.Generic;
using System.Data;
using Microsoft.SharePoint;
using PS.WebParts.Rollup.Query;
using PS.WebParts.Rollup.DAL.OM;
using Membership = System.Web.Security.Membership;

namespace PS.WebParts.Rollup.Query
{
    public class PSSiteDataQuery : IDisposable
    {
        private PSSiteDataListScope lists = new PSSiteDataListScope();
        private List<string> fields = new List<string>();
        private readonly ICacheProvider listCacheProvider = CacheSingleton.Instance;
        private string query;
        private uint startingRow;
        private uint rowLimit;
        private PSSiteDataSecuritySettings security = new PSSiteDataSecuritySettings();
        private PSSiteDataRollupOptions options = new PSSiteDataRollupOptions();
        private readonly GenericDAL dal = new GenericDAL();
        private readonly Dictionary<string, string> fieldMatches = new Dictionary<string, string>();

        //private readonly string dbName = "WSS_Content";
        public string ContentTypeV14Filter = string.Empty;

        public PSSiteDataListScope Lists
        {
            get { return lists; }
            set { lists = value; }
        }

        public List<string> Fields
        {
            get { return fields; }
            set { fields = value; }
        }

        public string Query
        {
            get { return query; }
            set { query = value; }
        }

        public uint StartingRow
        {
            get { return startingRow; }
            set { startingRow = value; }
        }

        public uint RowLimit
        {
            get { return rowLimit; }
            set { rowLimit = value; }
        }

        public PSSiteDataSecuritySettings Security
        {
            get { return security; }
            set { security = value; }
        }

        public PSSiteDataRollupOptions Options
        {
            get { return options; }
            set { options = value; }
        }

        public Dictionary<string, string> FieldMatches
        {
            get { return fieldMatches; }
        }

        public PSSiteDataQuery(ICacheProvider listCacheProvider)
        {
            this.listCacheProvider = listCacheProvider;
        }

        public DataTable GetRollupData(SPSite site, bool webAppLevel)
        {
            if (webAppLevel)
            {
                dal.SetWebAppLevel();
            }

            return GetRollupData(site);
        }

        public DataTable GetRollupData(SPSite site)
        {
            DataTable res;
            using (SPWeb rootWeb = site.RootWeb)
            {
                res = GetRollupData(rootWeb, true);
            }
            return res;
        }

        public static bool IsFBAAccount(string userName)
        {
            //            If (username.Contains(:)) // Looks like an FBA name

            //Try{

            //Membership.Provider.Current.GetUser(username, false);

            ////account is an FBA user

            //}catch(MembershipException)

            //{

            //                // account is not membership prov

            //}
            bool res = false;
            if (userName.Contains(":"))
            {
                try
                {
                    Membership.Provider.GetUser(userName, false);
                    res = true;
                }
                catch (Exception)
                {
                }
            }
            return res;
        }

        public DataTable GetRollupData(SPWeb web, bool isRecursive)
        {
            QueryParams.Instance.Reset();

            DataTable res = new DataTable("Rollup data");

            int userID = web.CurrentUser.ID;
            bool isAdmin = web.CurrentUser.IsSiteAdmin;

            bool isFBAUser = IsFBAAccount(web.CurrentUser.LoginName);

            QueryParams.Instance.UserId = userID;
            QueryParams.Instance.UserName = web.CurrentUser.Name;

            //#if DEBUG   //Testing purposes only
            //            dal.SetWebAppLevel();
            //#endif
            dal.ContentTypeV14Filter = this.ContentTypeV14Filter;

            dal.Connect(web.Site.ContentDatabase.DatabaseConnectionString, listCacheProvider);

            List<string> contentDBs = new List<string>();

            foreach (var contentDb in web.Site.WebApplication.ContentDatabases)
            {
                contentDBs.Add(contentDb.Name);
            }

            dal.CreateViewsInTmp(contentDBs.ToArray());

            RollUpQueryWebScope scope = isRecursive ? RollUpQueryWebScope.Recursive : RollUpQueryWebScope.Web;

            //List<DBList> accessibleLists

            WhereTerm[] whereTermArray = GetListScope(scope, web.ID, web.ServerRelativeUrl);

            string listSchema = dal.GetAccessibleLists(userID, whereTermArray, isAdmin, isFBAUser);

            if (!string.IsNullOrEmpty(listSchema))
            {
                Dictionary<string, string> userFields;
                Dictionary<string, LookupField> lookupFields;
                Dictionary<string, LookupField> lookupMultiFields;

                Dictionary<string, string> allFields = dal.ParseFields(listSchema, out userFields,
                                                                       out lookupFields, out lookupMultiFields);

                QueryParams.Instance.UserFields = userFields;
                //QueryParams.Instance.UserMultiFields = userMultiFields;

                GetRollupDataImpl(ref res, userID, allFields, userFields,
                                  lookupFields, lookupMultiFields, isAdmin, isFBAUser);
            }

            return res;
        }

        private void GetRollupDataImpl(ref DataTable res, int userID,
            Dictionary<string, string> allFields, Dictionary<string, string> userFields,
            Dictionary<string, LookupField> lookupFields, Dictionary<string, LookupField> lookupMutliFields,
            bool isAdmin, bool isFBAUser)
        {
            if (fields != null && fields.Count > 0)
                fields.Sort();

            Dictionary<string, string> selectedFields = new Dictionary<string, string>();

            foreach (string field in fields)
            {
                if (allFields.ContainsKey(field))
                {
                    selectedFields.Add(field, allFields[field]);
                }
            }

            dal.SetOptions(Options, startingRow, rowLimit, security, query, allFields, userFields,
                           lookupFields, lookupMutliFields, fieldMatches, isAdmin, isFBAUser);

            res = dal.GetRollupGeneralImpl(userID, selectedFields);
        }

        /// <summary>
        /// Returns array of WhereTerm, which filters available lists
        /// </summary>
        /// <param name="scope">Scope</param>
        /// <param name="spObjID">ID of SPSite or SPWeb</param>
        /// <returns></returns>
        /// <param name="webUrl">relative web URL</param>
        private WhereTerm[] GetListScope(RollUpQueryWebScope scope, Guid spObjID, string webUrl)
        {
            List<WhereTerm> res = new List<WhereTerm>();

            if (lists != null)
            {
                if (lists.Ids != null && lists.Ids.Count > 0)
                {
                    res.Add(WhereTerm.CreateIn(SqlExpression.Field("tp_ID",
                                                                 DAL.AllListsTable),
                                             SqlConstantCollection.FromList(lists.Ids.ConvertAll<string>(
                                               delegate(Guid id)
                                               {
                                                   return id.ToString("D");
                                               }
                                               ))));
                }
                else if (!string.IsNullOrEmpty(lists.ListTitle))
                {
                    res.Add(WhereTerm.CreateCompare(SqlExpression.Field("tp_Title", DAL.AllListsTable),
                                                    SqlExpression.Constant(SqlConstant.String(lists.ListTitle)),
                                                    CompareOperator.Equal));
                }
                else if (lists.ServerTemplate >= 100)
                {
                    res.Add(WhereTerm.CreateCompare(SqlExpression.Field("tp_ServerTemplate", DAL.AllListsTable),
                                            SqlExpression.Constant(SqlConstant.Number(lists.ServerTemplate)),
                                            CompareOperator.Equal));
                }

                if (!lists.Hidden)
                {
                    res.Add(
                        WhereTerm.CreateCompare(
                            SqlExpression.Raw(string.Format("[{0}].[tp_Flags] & 0x100", DAL.AllListsTable.RefName)),
                            SqlExpression.Raw("0x0"),
                            CompareOperator.Equal));
                }

                res.Add(dal.GetScopeConstrait(scope, spObjID, webUrl));
            }

            return res.ToArray();
        }

        public void Dispose()
        {
            dal.Dispose();
        }
    }
}