using System;
using System.Data;
using NUnit.Framework;
//#if TESTS
namespace PS.WebParts.Rollup.Test
{
    [TestFixture]
    public class TestCrossSiteDataSource
    {
        [Test]
        public  void GetCrossSiteData()
        {
            CrossSiteDataSource ds = new CrossSiteDataSource();
            ds.Scope = RollUpQueryWebScope.Recursive;
            ds.Lists = "100";
            ds.ListFields = "Title";
            ds.QueryRowLimit = "10";
            ds.ContentType = "Item";
            DataTable data = ds.GetRollUpData("http://wss-server/");
            
            Assert.IsNotNull(data);
            Assert.Greater(data.Rows.Count, 1, "Row count");
            Console.WriteLine("rows count: {0}", data.Rows.Count);
        }

        [Test]
        public void GetCrossSiteDataByContentType()
        {
            CrossSiteDataSource ds = new CrossSiteDataSource();
            ds.Scope = RollUpQueryWebScope.Recursive;
            ds.Lists = "100";
            ds.ListFields = "Title,Modified";
            //ds.ListFields = "ShipID";
            //ds.QueryRowLimit = "10";
            ds.ContentType = "Item";
            DataTable data = ds.GetRollUpData("http://wss-server/rollup/");
            Assert.IsNotNull(data);
            Assert.Greater(data.Rows.Count, 1, "Row count");
            Console.WriteLine("rows count: {0}", data.Rows.Count);            
        }

        [Test]
        public void GetGetListDetails()
        {
            CrossSiteDataSource ds = new CrossSiteDataSource();
            ds.Scope = RollUpQueryWebScope.Recursive;
            ds.Lists = "*";
            ds.ListFields = "Title,FirstName,Email";
            ds.IncludeListData = true;
            DataTable dt = ds.GetRollUpData("/Programs");
            DataSet data = new DataSet();
            data.Tables.Add(dt);
                
            Assert.IsNotNull(data);
            Console.WriteLine(data.GetXml());
            Assert.AreNotEqual(0, data.Tables.Count, "Table Count");
            DataTable table = data.Tables[0];
            Assert.Greater(table.Rows.Count, 1, "Row count");
            DataColumnCollection columns = table.Columns;
            Assert.IsNotNull( columns["_SiteTitle"], "site title" );
            Assert.IsNotNull(columns["_SiteUrl"], "site url");
            Assert.IsNotNull(columns["_ListTitle"], "list title");
            Assert.IsNotNull(columns["_ListUrl"], "list url");
            Assert.IsNotNull(columns["_ItemId"], "item id");
        }
    }
}
//#endif