using System;
using System.Collections.Generic;
using System.Data;
using NUnit.Framework;
using PS.WebParts.Rollup.Core;

namespace PS.WebParts.Rollup.Test
{

    [TestFixture]
    public class TestJoinProcessor
    {

        [Test]
        public void TestUnion()
        {
            List<DataTable> tables = SetupTablesForUnion();

            DataTable res = JoinProcessor.Union(tables);

            DataRowCollection resRows = res.Rows;

            Assert.AreEqual(4, resRows.Count);
            Assert.AreEqual(4, res.Columns.Count);

            Assert.AreEqual("1", resRows[0][0]);
            Assert.AreEqual("a1", resRows[0][1]);
            Assert.AreEqual("b1", resRows[0][2]);
            Assert.AreSame(DBNull.Value, resRows[0][3]);

            Assert.AreEqual("2", resRows[1][0]);
            Assert.AreEqual("a2", resRows[1][1]);
            Assert.AreEqual("b2", resRows[1][2]);
            Assert.AreSame(DBNull.Value, resRows[1][3]);

            Assert.AreEqual("1", resRows[2][0]);
            Assert.AreEqual("a2_1", resRows[2][1]);
            Assert.AreSame(DBNull.Value, resRows[2][2]);
            Assert.AreEqual("c1", resRows[2][3]);

            Assert.AreEqual("2", resRows[3][0]);
            Assert.AreEqual("a2_3", resRows[3][1]);
            Assert.AreSame(DBNull.Value, resRows[3][2]);
            Assert.AreEqual("c3", resRows[3][3]);

        }

        private static List<DataTable> SetupTablesForUnion()
        {
            DataTable t1;
            DataTable t2;

            t1 = new DataTable("T1");
            t2 = new DataTable("T2");

            t1.Columns.Add("ID");
            t1.Columns.Add("A");
            t1.Columns.Add("B");


            t2.Columns.Add("ID");
            t2.Columns.Add("A");
            t2.Columns.Add("C");

            DataRowCollection baseRows = t1.Rows;
            baseRows.Add("1", "a1", "b1");
            baseRows.Add("2", "a2", "b2");

            t2.Rows.Add("1", "a2_1", "c1");
            t2.Rows.Add("2", "a2_3", "c3");

            List<DataTable> tables = new  List<DataTable>(2);
            tables.Add(t1);
            tables.Add(t2);
            return tables;
        }

        [Test]
        public void TestInnerJoin()
        {
            DataSet ds = new DataSet();

            DataTable t1;
            DataTable t2;
            
            DataRowCollection baseRows = SetupTables(ds, out t1, out t2);

            DataRelation join = new DataRelation("blah", t1.Columns[0], t2.Columns[0]);
            ds.Relations.Add(join);
            
            List<string> excStrings = new  List<string>();
            excStrings.Add("ID");
            
            JoinProcessor.InnerJoin(t1, t2, join, excStrings);
            
            Assert.AreEqual(3, baseRows.Count);
            Assert.AreEqual(5, t1.Columns.Count);
            
            Assert.AreEqual("1", baseRows[0][0]);
            Assert.AreEqual("a1", baseRows[0][1]);
            Assert.AreEqual("b1", baseRows[0][2]);
            Assert.AreEqual("c1", baseRows[0][3]);
            Assert.AreEqual("d1", baseRows[0][4]);


            Assert.AreEqual("1", baseRows[1][0]);
            Assert.AreEqual("a1", baseRows[1][1]);
            Assert.AreEqual("b1", baseRows[1][2]);
            Assert.AreEqual("c2", baseRows[1][3]);
            Assert.AreEqual("d2", baseRows[1][4]);


            Assert.AreEqual("2", baseRows[2][0]);
            Assert.AreEqual("a2", baseRows[2][1]);
            Assert.AreEqual("b2", baseRows[2][2]);
            Assert.AreEqual("c3", baseRows[2][3]);
            Assert.AreEqual("d3", baseRows[2][4]);
        }

        private static DataRowCollection SetupTables(DataSet ds, out DataTable t1, out DataTable t2)
        {
            t1 = new DataTable("T1");
            t2 = new DataTable("T2");

            ds.Tables.Add(t1);
            ds.Tables.Add(t2);

            t1.Columns.Add("ID");
            t1.Columns.Add("A");
            t1.Columns.Add("B");


            t2.Columns.Add("ID");
            t2.Columns.Add("C");
            t2.Columns.Add("D");

            DataRowCollection baseRows = t1.Rows;
            baseRows.Add("1", "a1", "b1");
            baseRows.Add("2", "a2", "b2");
            baseRows.Add("3", "a3", "b3");
            baseRows.Add("4", "a4", "b4");

            t2.Rows.Add("1", "c1", "d1");
            t2.Rows.Add("1", "c2", "d2");
            t2.Rows.Add("2", "c3", "d3");
            return baseRows;
        }


        [Test]
        public void TestLeftJoin()
        {
            DataSet ds = new DataSet();

            DataTable t1;
            DataTable t2;

            DataRowCollection baseRows = SetupTables(ds, out t1, out t2);

            DataRelation join = new DataRelation("blah", t1.Columns[0], t2.Columns[0]);
            ds.Relations.Add(join);

            List<string> excStrings = new List<string>();
            excStrings.Add("ID");

            JoinProcessor.LeftJoin(t1, t2, join, excStrings);

            Assert.AreEqual(5, baseRows.Count);
            Assert.AreEqual(5, t1.Columns.Count);

            Assert.AreEqual("1", baseRows[0][0]);
            Assert.AreEqual("a1", baseRows[0][1]);
            Assert.AreEqual("b1", baseRows[0][2]);
            Assert.AreEqual("c1", baseRows[0][3]);
            Assert.AreEqual("d1", baseRows[0][4]);


            Assert.AreEqual("1", baseRows[1][0]);
            Assert.AreEqual("a1", baseRows[1][1]);
            Assert.AreEqual("b1", baseRows[1][2]);
            Assert.AreEqual("c2", baseRows[1][3]);
            Assert.AreEqual("d2", baseRows[1][4]);


            Assert.AreEqual("2", baseRows[2][0]);
            Assert.AreEqual("a2", baseRows[2][1]);
            Assert.AreEqual("b2", baseRows[2][2]);
            Assert.AreEqual("c3", baseRows[2][3]);
            Assert.AreEqual("d3", baseRows[2][4]);


            Assert.AreEqual("3", baseRows[3][0]);
            Assert.AreEqual("a3", baseRows[3][1]);
            Assert.AreEqual("b3", baseRows[3][2]);
            Assert.AreSame(DBNull.Value, baseRows[3][3]);
            Assert.AreSame(DBNull.Value, baseRows[3][4]);


            Assert.AreEqual("4", baseRows[4][0]);
            Assert.AreEqual("a4", baseRows[4][1]);
            Assert.AreEqual("b4", baseRows[4][2]);
            Assert.AreSame(DBNull.Value, baseRows[4][3]);
            Assert.AreSame(DBNull.Value, baseRows[4][4]);
        }
       
    }
}

