
using System;
using System.Data;
using System.IO;
using System.Text;
using System.Xml;
using Microsoft.SharePoint;
using NUnit.Framework;
using PS.WebParts.Rollup.Core;
//#if TESTS
namespace PS.WebParts.Rollup.Test
{
    [TestFixture]
    public class TestUtil
    {
        [Test]
        public void ResolveRelUrl()
        {
            string cur = "/a/b/c/d/";
            string path = "../../";
            string res = "/a/b";
            
            Assert.AreEqual(res, Util.ScopeUp(cur, path));
        }

        [Test]
        public void ResolveRelUrlTrailingName()
        {
            string cur = "/a/b/c/d/";
            string path = "../../Blah/";
            string res = "/a/b/Blah";

            Assert.AreEqual(res, Util.ScopeUp(cur, path));
        }

        [Test]
        public void ResolveRelUrl2()
        {
            string cur = "/a/b/c/d";
            string path = "../";
            string res = "/a/b/c";

            Assert.AreEqual(res, Util.ScopeUp(cur, path));
        }

        [Test]
        public void ResolveRelUrl3()
        {
            string cur = "/a/b/c/d";
            string path = "../../../../../";
            string res = "/";

            Assert.AreEqual(res, Util.ScopeUp(cur, path));
        }

        [Test]
        public void ScopeDown()
        {
            string cur = "/a/b/c/d";
            string path = "./";
            string res = "/a/b/c/d";

            Assert.AreEqual(res, Util.ScopeDown(cur, path));
        }

        [Test]
        public void ScopeDown2()
        {
            string cur = "/a/b/c/d";
            string path = "./Blah";
            string res = "/a/b/c/d/Blah";

            Assert.AreEqual(res, Util.ScopeDown(cur, path));
            
            path = "Blah";
            res = "/a/b/c/d/Blah";

            Assert.AreEqual(res, Util.ScopeDown(cur, path));
            
        }


        [Test]
        public void ResolveRelUrl5()
        {
            string cur = "/a/b/c/d";
            string path = "./Blah";
            string res = "/a/b/c/d/Blah";

            Assert.AreEqual(res, Util.ResolveRelUrl(cur, path));

            path = "Blah";
            res = "/a/b/c/d/Blah";

            Assert.AreEqual(res, Util.ResolveRelUrl(cur, path));

            res = "/1235/1234";
            Assert.AreEqual(res, Util.ResolveRelUrl(cur, res));

        }

        [Test]
        public void SetColumnsType()
        {
            DataTable dt = new DataTable("TestTable");
            dt.Columns.Add("ID", typeof (string));
            dt.Columns.Add("CreationDate", typeof (string));
            dt.Columns.Add("Name", typeof (string));
            dt.Columns.Add("Guid", typeof (string));

            dt.Columns["ID"].ExtendedProperties.Add(FieldInfo.ExtPropKey, SPFieldType.Integer);
            dt.Columns["CreationDate"].ExtendedProperties.Add(FieldInfo.ExtPropKey, SPFieldType.DateTime);
            dt.Columns["Name"].ExtendedProperties.Add(FieldInfo.ExtPropKey, SPFieldType.Text);
            dt.Columns["Guid"].ExtendedProperties.Add(FieldInfo.ExtPropKey, SPFieldType.Guid);
            
            for (int i = 0; i < 100; ++i)
            {
                DataRow dr = dt.NewRow();
                dr["ID"] = i.ToString();
                dr["CreationDate"] = (DateTime.Today + new TimeSpan(i,0,0,0)).ToString();
                dr["Name"] = "Name" + i.ToString();
                dr["Guid"] = Guid.NewGuid().ToString();
                dt.Rows.Add(dr);
            }

            DataTable typedTable = Util.CreateTypedTable(dt, FieldInfo.ExtPropKey);

            Assert.AreEqual(typedTable.Columns.Count, dt.Columns.Count);
            Assert.AreEqual(100, dt.Rows.Count);
            Assert.AreEqual(typedTable.Rows.Count, dt.Rows.Count);

            Assert.IsTrue(typedTable.Columns["ID"].DataType == typeof(int));
            Assert.IsTrue(typedTable.Columns["CreationDate"].DataType == typeof(DateTime));
            Assert.IsTrue(typedTable.Columns["Name"].DataType == typeof(string));
            Assert.IsTrue(typedTable.Columns["Guid"].DataType == typeof(string));

            for(int i = 0; i<typedTable.Rows.Count; ++i)
            {
                DataRow r1 = typedTable.Rows[i];
                DataRow r2 = dt.Rows[i];
                Assert.AreEqual(r1["ID"].ToString(), r2["ID"]);
                Assert.AreEqual(r1["CreationDate"].ToString(), r2["CreationDate"]);
                Assert.AreEqual(r1["Name"], r2["Name"]);
                Assert.AreEqual(r1["Guid"], r2["Guid"]);
            }
        }

        private readonly string psNamespace = "http://schemas.portalsolutions.net/Rollup/DataView/";

        private string ReplaseGroupFunction(string xsl)
        {
            XmlDocument xDoc = new XmlDocument();
            xDoc.LoadXml(xsl);
            XmlNamespaceManager xnm = new XmlNamespaceManager(xDoc.NameTable);
            xnm.AddNamespace("xsl", "http://www.w3.org/1999/XSL/Transform");
            XmlNode root = xDoc.SelectSingleNode("/Xsl");
            XmlNode node = xDoc.SelectSingleNode("//xsl:stylesheet", xnm);
            XmlAttribute attr = xDoc.CreateAttribute("xmlns:ps");
            attr.Value = psNamespace;
            node.Attributes.Append(attr);
            using(StringWriter sw = new StringWriter(new StringBuilder()))
            {
                xDoc.Save(sw);
                string res = sw.ToString();
                res = res.Replace("ddwrt:NameChanged(", "ps:NameChanged(");
                return res;
            }
        }

        [Test]
        public void XslEditTest()
        {
            string xsl =
                @"<Xsl>

					<xsl:stylesheet xmlns:x='http://www.w3.org/2001/XMLSchema' 
xmlns:d='http://schemas.microsoft.com/sharepoint/dsp' 
version='1.0' exclude-result-prefixes='xsl msxsl ddwrt' 
xmlns:ddwrt='http://schemas.microsoft.com/WebParts/v2/DataView/runtime' 
xmlns:asp='http://schemas.microsoft.com/ASPNET/20' 
xmlns:__designer='http://schemas.microsoft.com/WebParts/v2/DataView/designer' 
xmlns:xsl='http://www.w3.org/1999/XSL/Transform' 
xmlns:msxsl='urn:schemas-microsoft-com:xslt' xmlns:SharePoint='Microsoft.SharePoint.WebControls' 
xmlns:ddwrt2='urn:frontpage:internal'>
</xsl:stylesheet>
</Xsl>";
            Console.WriteLine(ReplaseGroupFunction(xsl));
        }

    }
}
//#endif