using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;
using System.Web.UI;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using NUnit.Framework;
using NUnit.Framework.SyntaxHelpers;
using PS.WebParts.Rollup.Query;
using PS.WebParts.Rollup.DAL.OM;
using PS.WebParts.Rollup.DAL.OM.Render;

namespace PS.WebParts.Rollup.Query.Tests
{
    [TestFixture]
    public class PSSiteDataQueryTest
    {
        [Test]
        public void GetRollupDataTest()
        {
            //TypedList
            using (SPSite site = new SPSite("http://localhost"))
            {
                using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                {
                    query.Fields.AddRange(new string[] {"ShipID", "RecordDate" });
                    query.Lists.ListTitle = "TypedList";
                    //query.Security.SecurityLevel
                    //query.Options.DatesInIso8601 = true;
                    query.Security.SecurityLevel = SecurityLevel.Item;

                    query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List | IncludeMetaDataOptions.Web;
                    query.Query = @"<Query>
   <OrderBy>
      <FieldRef Name=""ShipID""/>
      <FieldRef Name=""RecordDate"" Ascending=""True""/>
      <FieldRef Name=""Blah"" Ascending=""False""/>
   </OrderBy>
</Query>";
                    DataTable res = query.GetRollupData(site, true);
                }

                using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                {
                    query.Fields.AddRange(new string[] { "ShipID", "RecordDate" });
                    query.Lists.ListTitle = "TypedList";
                    //query.Security.SecurityLevel
                    //query.Options.DatesInIso8601 = true;
                    query.Security.SecurityLevel = SecurityLevel.Item;

                    query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List | IncludeMetaDataOptions.Web;

                    DataTable res = query.GetRollupData(site);
                }
            }
        }

        [Test]
        public void MemoryUsageTest()
        {
            using(SPSite site = new SPSite("http://localhost/"))
            { 
                //for (int i = 0; i < 200; ++i)
                //{
                //    using (SPWeb web = site.OpenWeb("/rollup"))
                //    {
                //        string name = "test" + (i + 1);
                //        web.Webs.Add(name, name, "", 1033, "rollup_query_test_site.stp", false, false);
                //    }
                //}

                using(SPWeb web  = site.OpenWeb("/rollup"))
                {

                    Console.WriteLine("total managed memory before query: {0} bytes", GC.GetTotalMemory(false));
                    Stopwatch sw = Stopwatch.StartNew();

                    //SPSiteDataQuery spQuery = new SPSiteDataQuery();
                    //spQuery.Lists = @"<Lists ServerTemplate=""107"" />";
                    //spQuery.ViewFields = @"<FieldRef Name=""_x0073_tr1"" /><FieldRef Name=""_x0073_tr2"" /><FieldRef Name=""_x0073_tr3"" /><FieldRef Name=""_x0073_tr4"" /><FieldRef Name=""_x0073_tr5"" />";
                    //spQuery.Webs = @"<Webs Scope=""Recursive"" />";
                    //DataTable dt = web.GetSiteData(spQuery);

                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        query.Fields.AddRange(new string[] { "str1", "str2", "str3", "str4", "str5" });
                        query.Lists.ListTitle = "TaskList";
                        //query.StartingRow = 20;
                        //query.RowLimit = 500;
                        //query.Security.SecurityLevel
                        query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List | IncludeMetaDataOptions.Web;
                        query.Query =
                            @"<Query>
   <OrderBy>
      <FieldRef Name=""_x0073_tr1"" Ascending=""True"" />
   </OrderBy>
</Query>";
                        DataTable res = query.GetRollupData(web, true);
                        //Assert.AreEqual(res.Rows.Count, 500);
                        //Assert.AreEqual(res.Rows.Count, query.RowLimit);
                    }
                    
                    sw.Stop();

                    Console.WriteLine("query time: {0} ms", sw.ElapsedMilliseconds);
                    Console.WriteLine("total managed memory after query: {0} bytes", GC.GetTotalMemory(false));
                }

                using (SPWeb web = site.OpenWeb("/rollup"))
                {

                    Console.WriteLine("total managed memory before query: {0} bytes", GC.GetTotalMemory(false));
                    Stopwatch sw = Stopwatch.StartNew();

                    //SPSiteDataQuery spQuery = new SPSiteDataQuery();
                    //spQuery.Lists = @"<Lists ServerTemplate=""107"" />";
                    //spQuery.ViewFields = @"<FieldRef Name=""_x0073_tr1"" /><FieldRef Name=""_x0073_tr2"" /><FieldRef Name=""_x0073_tr3"" /><FieldRef Name=""_x0073_tr4"" /><FieldRef Name=""_x0073_tr5"" />";
                    //spQuery.Webs = @"<Webs Scope=""Recursive"" />";
                    //DataTable dt = web.GetSiteData(spQuery);

                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        query.Fields.AddRange(new string[] { "str1", "str2", "str3", "str4", "str5" });
                        query.Lists.ListTitle = "TaskList";
                        //query.StartingRow = 20;
                        //query.RowLimit = 500;
                        //query.Security.SecurityLevel
                        query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List | IncludeMetaDataOptions.Web;
                        DataTable res = query.GetRollupData(web, true);
                        //Assert.AreEqual(res.Rows.Count, 500);
                        //Assert.AreEqual(res.Rows.Count, query.RowLimit);
                        }

                    sw.Stop();

                    Console.WriteLine("query time: {0} ms", sw.ElapsedMilliseconds);
                    Console.WriteLine("total managed memory after query: {0} bytes", GC.GetTotalMemory(false));
                }
            }
        }

        //private const int TOKEN_READ = 0x00020008;
        //private const int TOKEN_IMPERSONATE = 0x0004;

        #region Domain groups

        public enum SID_NAME_USE
        {
            SidTypeAlias = 0x4,
            SidTypeComputer = 0x9,
            SidTypeDeletedAccount = 0x6,
            SidTypeDomain = 0x3,
            SidTypeGroup = 0x2,
            SidTypeInvalid = 0x7,
            SidTypeUnknown = 0x8,
            SidTypeUser = 0x1,
            SidTypeWellKnownGroup = 0x5
        }


        private List<string> GetSecurityGroupsForCurrentUser()
        {
            List<string> result = new List<string>();

            int tokenInfoLength = 0;
            //byte[] token = user.GetBinaryId();
            //IntPtr tokenPtr = Marshal.AllocHGlobal(token.Length);
            //Marshal.Copy(token, 0, tokenPtr, token.Length);

            IntPtr threadToken = IntPtr.Zero;
            bool res = OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, out threadToken);
            if (!res)
            {
                int errCode = Marshal.GetLastWin32Error();
                return null;
            }
            GetTokenInformation(threadToken,
                                      TOKEN_INFORMATION_CLASS.TokenGroups,
                                      IntPtr.Zero,
                                      tokenInfoLength,
                                      out tokenInfoLength);

            IntPtr tokenInfo = Marshal.AllocHGlobal(tokenInfoLength);
            res = GetTokenInformation(threadToken,
                                      TOKEN_INFORMATION_CLASS.TokenGroups,
                                      tokenInfo,
                                      tokenInfoLength,
                                      out tokenInfoLength);
            if (res)
            {
                TOKEN_GROUPS groups = (TOKEN_GROUPS)Marshal.PtrToStructure(tokenInfo, typeof(TOKEN_GROUPS));
                //SID_AND_ATTRIBUTES[] attribs = new SID_AND_ATTRIBUTES[groups.GroupCount];

                //IntPtr ptr = Marshal.UnsafeAddrOfPinnedArrayElement(groups.Groups, 0);
                //for(int i = 0; i<groups.GroupCount; ++i)
                //{
                //    ptr = new IntPtr(((int) ptr) + i*Marshal.SizeOf(typeof(SID_AND_ATTRIBUTES)));
                //    //attribs[i] = (SID_AND_ATTRIBUTES)Marshal.PtrToStructure(Marshal.UnsafeAddrOfPinnedArrayElement(attribs, i), typeof(SID_AND_ATTRIBUTES) );
                //    attribs[i] = (SID_AND_ATTRIBUTES)Marshal.PtrToStructure(ptr, typeof (SID_AND_ATTRIBUTES));

                //}

                //Marshal.Copy(groups.Groups, Marshal.StructureToPtr(attribs, 0, (int)groups.GroupCount);
                //Marshal.
                //TOKEN_USER groups = (TOKEN_USER)Marshal.PtrToStructure(tokenInfo, typeof(TOKEN_USER));
                //string account = LookupAccountByStringSid(groups.User.Sid);

                for (int i = 0; i < groups.GroupCount; i++)
                {
                    SID_AND_ATTRIBUTES group = groups.Groups[i];
//IntPtr pstr = IntPtr.Zero;
                    //bool ok = ConvertSidToStringSid(group.Sid, out pstr);
                    //if (ok)
                    //{
                    //    string sidstr = Marshal.PtrToStringAuto(pstr);
                    //    result.Add(sidstr);
                    //    LocalFree(pstr);
                    //}

                    try
                    {
                        string account = LookupAccountByStringSid(group.Sid);
                        result.Add(account);
                    }
                    catch (Win32Exception)
                    {
                    }
                }
            }
            else
            {
                int errCode = Marshal.GetLastWin32Error();
            }

            //Marshal.FreeHGlobal(tokenPtr);

            return result;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct SID_AND_ATTRIBUTES
        {
            public IntPtr Sid;
            public int Attributes;
        }

        enum TOKEN_INFORMATION_CLASS
        {
            TokenUser = 1,
            TokenGroups,
            TokenPrivileges,
            TokenOwner,
            TokenPrimaryGroup,
            TokenDefaultDacl,
            TokenSource,
            TokenType,
            TokenImpersonationLevel,
            TokenStatistics,
            TokenRestrictedSids,
            TokenSessionId,
            TokenGroupsAndPrivileges,
            TokenSessionReference,
            TokenSandBoxInert,
            TokenAuditPolicy,
            TokenOrigin
        }

        //public struct TOKEN_PRIVILEGES
        //{
        //    public UInt32 PrivilegeCount;
        //    [MarshalAs(UnmanagedType.ByValArray)]
        //    public SID_AND_ATTRIBUTES[] Privileges;
        //}

        [StructLayout(LayoutKind.Sequential)]
        public struct TOKEN_GROUPS
        {
            public uint GroupCount;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 1000)]
            public SID_AND_ATTRIBUTES[] Groups;
            //public IntPtr Groups;
        }
        
        //public struct TOKEN_USER
        //{
        //    public SID_AND_ATTRIBUTES User;
        //}

        [DllImport("advapi32.dll", SetLastError = true)]
        static extern bool GetTokenInformation(
            IntPtr TokenHandle,
            TOKEN_INFORMATION_CLASS TokenInformationClass,
            IntPtr TokenInformation,
            int TokenInformationLength,
            out int ReturnLength);

        //[DllImport("advapi32", CharSet = CharSet.Auto, SetLastError = true)]
        //static extern bool ConvertSidToStringSid(
        //    IntPtr pSID,
        //    out IntPtr ptrSid);

        //[DllImport("kernel32.dll")]
        //static extern IntPtr LocalFree(IntPtr hMem);

        //[DllImport("advapi32.dll", CharSet = CharSet.Unicode, SetLastError = true)]
        //internal static extern bool OpenThreadToken(IntPtr ThreadHandle, 
        //    uint DesiredAccess, bool OpenAsSelf, out IntPtr TokenHandle);

        [DllImport("advapi32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool OpenProcessToken(IntPtr ProcessHandle,
            uint DesiredAccess, out IntPtr TokenHandle);

        //[DllImport("kernel32.dll", SetLastError = true)]
        //private static extern IntPtr GetCurrentThread();

        [DllImport("kernel32.dll")]
        static extern IntPtr GetCurrentProcess();

        [DllImport("advapi32.dll", CharSet = CharSet.Unicode, SetLastError = true, EntryPoint = "LookupAccountSid")]
        internal static extern bool LookupAccountSidNative(string SystemName, IntPtr Sid, StringBuilder Name, 
            ref int cbName, StringBuilder DomainName, 
            ref int cbDomainName, ref SID_NAME_USE peUse);
 
        private const int TOKEN_QUERY = 0x0008;

        //private static bool LookupAccountSid(string SystemName, IntPtr Sid, StringBuilder Name,
        //    ref int cbName, StringBuilder DomainName,
        //    ref int cbDomainName, ref SID_NAME_USE peUse)
        //{
        //    return LookupAccountSidNative(SystemName, Sid, Name, ref cbName, DomainName, ref cbDomainName, ref peUse);
        //}

        internal static string LookupAccountByStringSid(IntPtr sid)
        {
            string res = null;

            SID_NAME_USE sidTypeUnknown = SID_NAME_USE.SidTypeUnknown;
            int capacity = 0x100;
            StringBuilder domainName = new StringBuilder(capacity);
            int num2 = 0x100;
            StringBuilder name = new StringBuilder(num2);
            if (!LookupAccountSidNative(null, sid, name, ref num2, domainName, ref capacity, ref sidTypeUnknown))
            {
                throw new Win32Exception(Marshal.GetLastWin32Error());
            }
            
            if (domainName.ToString().ToUpperInvariant() == "BUILTIN")
            {
                domainName = new StringBuilder(Environment.MachineName);
            }
            res = domainName + @"\" + name;
        
            return res;
        }
        #endregion

        [Test]
        public void DomainGroupsTest()
        {
            
            //GetSecurityGroupsForCurrentUser();
            for (int i = 0; i < 100; ++i )
                CurrentUser.GetDomainGroups().ForEach(Console.WriteLine);
        }

        [Test]
        public void QueryTest()
        {
            using (SPSite site = new SPSite("http://localhost/"))
            {
                using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                {
                    query.Fields.AddRange(new string[] {"ShipID", "RecordDate"});
                    query.Lists.ListTitle = "TypedList";
                    //query.Security.SecurityLevel
                    //query.Options.DatesInIso8601 = true;
                    query.Security.SecurityLevel = SecurityLevel.Item;

                    query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List | IncludeMetaDataOptions.Web;
                    query.Query =
                        @"
    <OrderBy>
        <FieldRef Name=""ShipID""/>
        <FieldRef Name=""RecordDate"" Ascending=""True""/>
        <FieldRef Name=""Blah"" Ascending=""False""/>
    </OrderBy>
    <Where>
        <Or>
            <Eq>
                <FieldRef Name=""ShipID""/>
                <Value Type=""Text"">15</Value>
            </Eq> 
            <Contains>
                <FieldRef Name=""ShipID""/>
                <Value Type=""Text"">5</Value>
            </Contains>
            <And>
                <Lt>
                    <FieldRef Name=""ShipID""/>
                    <Value Type=""Text"">78</Value>
                </Lt>
                <Geq>
                    <FieldRef Name=""ShipID""/>
                    <Value Type=""Text"">18</Value>
                </Geq>
            </And>
        </Or>
    </Where>
";
                    //query.Query = @"<OrderBy><FieldRef Name='ShipID'/></OrderBy>";

                    DataTable res = query.GetRollupData(site, true);
                }
            }
        }

        [Test]
        public void QueryComplexValueTest()
        {
            //using (SPSite site = new SPSite("http://localhost"))
            //{
            //    using (SPWeb web = site.OpenWeb("rollup/test1"))
            //    {
            //        using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
            //        {
            //            query.Fields.AddRange(new string[] { "Title", "AssignedTo", "str1", "str2" });
            //            query.Lists.ListTitle = "TaskList";
            //            query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List |
            //                                            IncludeMetaDataOptions.Web;

            //            query.Query =
            //                @"<Where><Eq><FieldRef Name='AssignedTo'/><Value Type='Integer'>_ItemID</Value></Eq></Where>";

            //            DataTable res = query.GetRollupData(web, true);
            //            Assert.AreEqual(res.Rows.Count, 1);
            //        }
            //    }
            //}

            using (SPSite site = new SPSite("http://localhost"))
            {
                using (SPWeb web = site.OpenWeb("rollup/test1"))
                {
                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        query.Fields.AddRange(new string[] {"Title", "AssignedTo", "str1", "str2"});
                        query.Lists.ListTitle = "TaskList";
                        query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List |
                                                        IncludeMetaDataOptions.Web;

                        query.Query =
                            @"<Where><Eq><FieldRef Name='AssignedTo'/><Value Type='Integer'><UserID/></Value></Eq></Where>";

                        DataTable res = query.GetRollupData(web, true);
                        //Assert.AreEqual(res.Rows.Count, 1);
                    }
                }
            }

            using (SPSite site = new SPSite("http://localhost"))
            {
                using (SPWeb web = site.OpenWeb("rollup/test1"))
                {
                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        query.Fields.AddRange(new string[] { "Title", "AssignedTo", "StartDate", "str1", "str2" });
                        query.Lists.ListTitle = "TaskList";
                        query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List |
                                                        IncludeMetaDataOptions.Web;

                        query.Query =
                            @"<Where><Lt><FieldRef Name='StartDate'/><Value Type='DateTime'><Today OffsetDays='-13'/></Value></Lt></Where>";

                        DataTable res = query.GetRollupData(web, true);
                        //Assert.AreEqual(res.Rows.Count, 4);
                        Assert.That(res.Rows.Count, Is.EqualTo(5));
                    }
                }
            }
        }

        [Test]
        public void FieldRefTest()
        {
            using(SPSite site = new SPSite("http://localhost/"))
            {
                using(SPWeb web =site.OpenWeb("/rollup"))
                {
                    SPList taskList = web.Lists["TypedList"];
                    SPField moderCommentsField = taskList.Fields.GetField("ContentTypeId");
                    string schema = moderCommentsField.SchemaXml;

                    Console.WriteLine("------fields------");
                    foreach (SPField field in taskList.Fields)
                    {
                        Console.WriteLine(field.InternalName);
                        Console.WriteLine(field.StaticName);
                        Console.WriteLine(field.SchemaXml);
                        Console.WriteLine();
                    }

                    Console.WriteLine("------content types------");
                    foreach (SPContentType contentType in taskList.ContentTypes)
                    {
                        Console.WriteLine(contentType.Name);
                        Console.WriteLine(contentType.Id);
                        Console.WriteLine(contentType.SchemaXml);
                    }
                }

                using (SPWeb web = site.OpenWeb("/rollup/test1"))
                {
                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        string[] fields = new string[] { "Title", "AssignedTo", "str1" };
                        query.Fields.AddRange(fields);
                        query.Lists.ListTitle = "TaskList";
                        query.Query =
                             @"<Query>
<OrderBy>
    <FieldRef Name='str1'/>
    </OrderBy>
</Query>";
                        DataTable res = query.GetRollupData(web, false);
                    }
                }
            }
        }

        [Test]
        public void UserTypeFieldTest()
        {
            using(SPSite site = new SPSite("http://localhost/"))
            {
                using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                {
                    query.Lists.ListTitle = "User Information List";
                    query.Lists.Hidden = true;
                    query.Fields.AddRange(new string[] { "Title", "UserName" });
                    query.Query =
                        @"<Query>
    <OrderBy>
        <FieldRef Name=""Title""/>        
    </OrderBy>
</Query>";
                    DataTable res = query.GetRollupData(site);
                }
            }
        }

        [Test]
        public void LookupFieldTest()
        {
            using(SPSite site = new SPSite("http://localhost/"))
            {
                Console.WriteLine("----- PSSiteDataQuery 1st iteration -----");

                using(SPWeb web = site.OpenWeb("/rollup/test1"))
                {
                    Console.WriteLine("total managed memory before query: {0} bytes", GC.GetTotalMemory(false));
                    Stopwatch sw = Stopwatch.StartNew();

                    using(PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        string[] fields = new string[] { "Title", "lookup_str2" };
                        query.Fields.AddRange(fields);
                        query.Lists.ListTitle = "TaskList";
                        query.Query = "<Where><Eq><FieldRef Name='lookup_str2'/><Value Type='Text'>text2_3</Value></Eq></Where>";
                        DataTable res = query.GetRollupData(web, false);
                        Assert.AreEqual(fields.Length + 2, res.Columns.Count); //+2 because of multi-valuee field and tp_Id 
                    }

                    sw.Stop();

                    Console.WriteLine("query time: {0} ms", sw.ElapsedMilliseconds);
                    Console.WriteLine("total managed memory after query: {0} bytes", GC.GetTotalMemory(false));
                }

//                Console.WriteLine("----- PSSiteDataQuery 2nd iteration -----");

//                using (SPWeb web = site.OpenWeb("/rollup/test1"))
//                {
//                    Console.WriteLine("total managed memory before query: {0} bytes", GC.GetTotalMemory(false));
//                    Stopwatch sw = Stopwatch.StartNew();

//                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
//                    {
//                        query.Fields.AddRange(new string[] { "Title", "lookup_str2" });
//                        query.Lists.ListTitle = "TaskList";
//                        DataTable res = query.GetRollupData(web, false);
//                    }

//                    sw.Stop();

//                    Console.WriteLine("query time: {0} ms", sw.ElapsedMilliseconds);
//                    Console.WriteLine("total managed memory after query: {0} bytes", GC.GetTotalMemory(false));
//                }

//                Console.WriteLine("----- SPSiteDataQuery -----");

//                using (SPWeb web = site.OpenWeb("/rollup/test1"))
//                {
//                    Console.WriteLine("total managed memory before query: {0} bytes", GC.GetTotalMemory(false));
//                    Stopwatch sw = Stopwatch.StartNew();

//                    SPSiteDataQuery spQuery = new SPSiteDataQuery();
//                    spQuery.Lists = @"<Lists>
//  <List ID=""0CBBE27C-E6AD-43BE-8CA9-5DA426CDAED7"" />
//</Lists>";
//                    spQuery.ViewFields = @"<FieldRef Name=""Title"" /><FieldRef Name=""lookup_str2"" />";
//                    DataTable dt = web.GetSiteData(spQuery);

//                    sw.Stop();

//                    Console.WriteLine("query time: {0} ms", sw.ElapsedMilliseconds);
//                    Console.WriteLine("total managed memory after query: {0} bytes", GC.GetTotalMemory(false));
//                }
            }
        }

        [Test]
        public void DataSourceFromPageTest()
        {
            Page p = new Page();
            p.AppRelativeVirtualPath = "~/rollup/default.aspx";
            string dsSource = @"<WpNs0:SqlRollupDataSource runat=""server"" Lists=""1603"" 
TopLevelSite=""/FundProperties"" FixLookups=""True"" DatesInISO8601=""True"" 
QueryRowLimit=""100"" ID=""DSC1"" Scope=""Recursive"" ContentType=""Property Info"" IncludeListData=""False"">
<Fields>
<FieldRef Name=""Title"" />

<FieldRef Name=""AuditProjectYear"" />

<FieldRef Name=""ProjectStatus"" />

<FieldRef Name=""ProjectDescription"" />

<FieldRef Name=""ProjectManager"" />

<FieldRef Name=""ProjectPriority"" />

</Fields>

<Query><Where><And><Eq><FieldRef Name='ProjectStatus'/><Value Type='Text'>Active</Value></Eq><Eq><FieldRef Name='ClientNumber'/><Value Type='Text'>[SPWeb.Properties[ClientNumber]]</Value></Eq></And></Where><OrderBy><FieldRef Name='Title'/></OrderBy></Query>

</WpNs0:SqlRollupDataSource>";
            SqlRollupDataSource sqlDs = p.ParseControl(dsSource) as SqlRollupDataSource;
            Console.WriteLine(sqlDs.Query);
        }

        [Test]
        public void SqlDatasourceTest()
        {
            SqlRollupDataSource sqlDs = new SqlRollupDataSource();
            sqlDs.TopLevelSite = "/rollup";
            sqlDs.Lists = "test_data";
            sqlDs.Scope = RollUpQueryWebScope.Recursive;
            //sqlDs.IncludeListData = true;
            sqlDs.Fields = @"<FieldRef Name='str1'  Match='item1_2' />";
            DataTable res = sqlDs.GetRollUpData("http://localhost");
        }

        [Test]
        public void GetListsTableNamesTest()
        {
           using(SPSite site = new SPSite("http://localhost/"))
           {
               using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
               {
                   query.Fields.AddRange(new string[] { "ShipID", "RecordDate" });
                   query.Lists.Ids.Add(new Guid("22ab2f97-93f3-4b4d-8101-7657b45bd3b5"));
                   query.Lists.Ids.Add(new Guid("5e34832a-bc6f-4776-b557-764f6f0e7673"));
                   //query.Security.SecurityLevel
                   //query.Options.DatesInIso8601 = true;
                   query.Security.SecurityLevel = SecurityLevel.Item;

                   query.Options.IncludeListData = IncludeMetaDataOptions.IDs | IncludeMetaDataOptions.List | IncludeMetaDataOptions.Web;

                   DataTable res = query.GetRollupData(site);
               }
           }
        }

        [Test]
        public void ReznickTest()
        {
            using (SPSite site = new SPSite("http://reznickfundsdev.portalsolutions.net"))
            {
                using (SPWeb web = site.OpenWeb("/FundProperties"))
                {
                    using (PSSiteDataQuery psq = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        string[] fields = new string[]
                            {
                                "PBCType", "PropertyWebId", "RequestStatus",
                                "PreviousDateReceived", "ReviewedBy", "ReviewDate", "PreparedBy", "PreparedDate",
                                "FinalDate", "ContentType"
                            };

                        //string[] fields = new string[]
                        //    {
                        //        "PBCType", "RequestStatus"
                        //    };

                        psq.Fields.AddRange(fields);

                        psq.Query =
                            "<Where><And><And><Eq><FieldRef Name='PBCType'/><Value Type='Text'>Tax Return</Value></Eq><Eq><FieldRef Name='ContentType'/><Value Type='Text'>PBC Request</Value></Eq></And><Eq><FieldRef Name='PropertyWebId'/><Value Type='Text'>3ADC9C78-DEC8-46D9-9064-1B64E58B7D68</Value></Eq></And></Where>";

                        psq.Lists.ServerTemplate = 1606;
                        psq.Options.IncludeListData = IncludeMetaDataOptions.IDs;

                        DataTable dt = psq.GetRollupData(web,true);
                        Assert.Greater(dt.Rows.Count, 0);
                    }
                    
                }
            }
        }

        [Test]
        public void AvTest()
        {
            for (int i = 0; i < 100; ++i)
            {
                Console.Write(i + "\t");
                int tokenInfoLength = 0;
                IntPtr threadToken;

                bool res = OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, out threadToken);
                if (res)
                {
                    //Console.WriteLine("threadToken = {0}", threadToken);
                    GetTokenInformation(threadToken,
                                        TOKEN_INFORMATION_CLASS.TokenGroups,
                                        IntPtr.Zero,
                                        tokenInfoLength,
                                        out tokenInfoLength);

                    IntPtr tokenInfo = Marshal.AllocHGlobal(tokenInfoLength);
                    //Console.WriteLine("tokenInfoLength = {0}", tokenInfoLength);

                    res = GetTokenInformation(threadToken,
                                              TOKEN_INFORMATION_CLASS.TokenGroups,
                                              tokenInfo,
                                              tokenInfoLength,
                                              out tokenInfoLength);

                    if (res)
                    {
                        TOKEN_GROUPS groups = (TOKEN_GROUPS) Marshal.PtrToStructure(tokenInfo, typeof (TOKEN_GROUPS));
                    }

                    Marshal.FreeHGlobal(tokenInfo);
                }
            }
        }

        //[Test]
        //public void GetOffsetTest()
        //{
        //    StorageDateOffset sdo = new StorageDateOffset();
        //    TimeSpan ts = sdo.GetOffset();
        //    Console.WriteLine(ts);
        //}

        [Test]
        public void CFFPMSTest()
        {
            using (SPSite site = new SPSite("http://ccfpms.portalsolutions.net"))
            {
                using (SPWeb web = site.OpenWeb("OpportunityHomepage"))
                {
                    using (PSSiteDataQuery psq = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        string[] fields = new string[]
                            {
                                "TaskTitle", "TaskStatus", "TaskDueDate",
                                "ContentType", "AssignedTo"
                            };

                        psq.Fields.AddRange(fields);

                        psq.Options.FixLookups = true;
                        psq.RowLimit = 100;

                        psq.Query = @"<Where>
<And>
<Eq><FieldRef Name='ContentType' />
<Value Type='Text'>Opportunity Tasks</Value></Eq>
<And>
<And>
<Eq><FieldRef Name='AssignedTo' />
<Value Type='Integer'><UserID/></Value></Eq>
<Lt><FieldRef Name='TaskDueDate' />
<Value Type='DateTime'><Today /></Value></Lt>
</And>
<Neq><FieldRef Name='TaskStatus' />
<Value Type='Text'>Completed</Value></Neq>
</And>
</And>
</Where>";
                        psq.Lists.ServerTemplate = 107;

                        DataTable dt = psq.GetRollupData(web, true);
                        Assert.Greater(dt.Rows.Count, 0);
                    }
                }
            }
        }

        [Test]
        public void SameTitleLookupFieldsTest()
        {
            using (SPSite site = new SPSite("http://jmtdev.portalsolutions.net"))
            {
                using (SPWeb web = site.OpenWeb("projectcenter"))
                {
                    using (PSSiteDataQuery psq = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        string[] fields = new string[]
                            {
                                "ProjectName", "ProjectNumber", "Contract",
                                "ContractType", "ClientName", "ProjectManager", "ProjectLocation",
                                "ContentType"
                            };

                        psq.Fields.AddRange(fields);

                        psq.Options.FixLookups = true;
                        psq.RowLimit = 100;

                        psq.Lists.ServerTemplate = 1601;

                        psq.Options.IncludeListData = IncludeMetaDataOptions.IDs;

                        psq.Query =
                            "<Where><Eq><FieldRef Name='ContentType'/><Value Type='Text'>Project Info</Value></Eq></Where>";


                        DataTable dt = psq.GetRollupData(web, true);
                        Assert.Greater(dt.Rows.Count, 0);
                    }
                }
            }
        }

        [Test]
        public void MutliValueFieldTest()
        {
            using(SPSite site = new SPSite("http://localhost/"))
            {
                using (SPWeb web = site.OpenWeb("rollup/test1"))
                {
                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        string[] fields = new string[]
                            { "Title", "lookup_choise" };
                        
                        query.Fields.AddRange(fields);
                        query.Lists.Ids.Add(new Guid("0CBBE27C-E6AD-43BE-8CA9-5DA426CDAED7"));
                        
                        DataTable dt = query.GetRollupData(web, false);
                        Assert.That(dt.Rows.Count, Is.EqualTo(5));
                    }
                }
            }
        }

        [Test]
        public void ConnectionTest()
        {
            string connectionString =
                @"Server=np:\\.\pipe\MSSQL$Microsoft##SSEE\sql\query;Database=WSS_Content;Trusted_Connection=yes;App=Windows SharePoint Services;Timeout=500";
            SqlConnection connection = new SqlConnection(connectionString);
            connection.Open();
            SqlCommand command = new SqlCommand("SELECT * FROM WSS_Content_555.dbo.AllLists", connection);
            using(SqlDataReader reader = command.ExecuteReader())
            {
                while(reader.Read())
                {
                    Console.WriteLine(reader.GetValue(0));
                }
            }
            connection.Close();
        }

        [Test]
        public void WebAppDBsTest()
        {
            using(SPSite site = new SPSite("http://localhost"))
            {
                List<string> contentDBs = new List<string>();

                foreach (var contentDb in site.WebApplication.ContentDatabases)
                {
                    contentDBs.Add(contentDb.Name);
                }

                DAL d = new GenericDAL();
                d.Connect(site.ContentDatabase.DatabaseConnectionString, CacheSingleton.Instance);
                d.SetWebAppLevel();
                d.CreateViewsInTmp(contentDBs.ToArray());
            }
        }

        [Test]
        public void SqlQueryTest()
        {
            var builder = new SqlConnectionStringBuilder();
            builder.DataSource = "(local)";
            builder.InitialCatalog = "TestDB";

            string sqlQuery = "SELECT * FROM dbo.Table1";

            //SqlDataSourceView dataSource = new SqlDataSourceView()
            
        }

        [Test]
        public void UserMultiTest()
        {
            using (SPSite site = new SPSite("http://localhost"))
            {
                using (SPWeb web = site.OpenWeb("rollup"))
                {
                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        query.Fields.Add("Title");
                        query.Lists.ListTitle = "multi_user";
                        DataTable dt = query.GetRollupData(web, false);
                        Assert.That(dt.Rows.Count, Is.EqualTo(3));
                    }
                }
            }
        }

        [Test]
        public void LookupMultiTest()
        {
            using (SPSite site = new SPSite("http://localhost"))
            {
                using (SPWeb web = site.OpenWeb("rollup"))
                {
                    using (PSSiteDataQuery query = new PSSiteDataQuery(CacheSingleton.Instance))
                    {
                        query.Fields.Add("Title");
                        query.Lists.ListTitle = "multi_user";
                        DataTable dt = query.GetRollupData(web, false);
                        Assert.That(dt.Rows.Count, Is.EqualTo(3));
                    }
                }
            }
        }
    }
}
