using System.Collections.Generic;
using NUnit.Framework;
using PS.WebParts.Rollup.Core;

namespace PS.WebParts.Rollup.Test
{
    [TestFixture]
    public class TestAggregateParser
    {
        [Test]
        public void TestParse()
        {
            string xml = @"<join left=""DSC1"" right=""DSC2"" on=""ID=ID"" type=""INNER"" correlate=""Web"">
                                <join right=""DSC3"" on=""ID=ID"" type=""INNER"" />
                                <join right=""DSC4"" on=""ID=ID"" type=""RIGHT"" />
                            </join><join right=""DSC5"" on=""ID=ID"" type=""RIGHT"" />";

            SetOperation root = AggregateParser.Parse(xml);

            List<SetOperation> ops = root.Children;
            Assert.AreEqual(2, ops.Count);
            SetOperation op = ops[0];
            Assert.IsInstanceOfType(typeof(Join), op);
            
            Join j = (Join) op;

            Assert.AreEqual(Join.CorrelationType.Web, j.Correlate);
            Assert.AreEqual("DSC1", j.BaseId);
            Assert.AreEqual("ID", j.BaseColumn);

            Assert.AreEqual("DSC2", j.JoineeId);
            Assert.AreEqual("ID", j.JoineeColumn);
            
            Assert.AreEqual(Join.Type.Inner, j.JoinType);
            
            Assert.AreEqual(2, j.Children.Count);

            Assert.IsInstanceOfType(typeof(Join), j.Children[0]);
            Join j1 = (Join) j.Children[1];
            j = (Join)j.Children[0];

            Assert.AreEqual("DSC1", j.BaseId);
            Assert.AreEqual("ID", j.BaseColumn);

            Assert.AreEqual("DSC3", j.JoineeId);
            Assert.AreEqual("ID", j.JoineeColumn);

            Assert.AreEqual(Join.Type.Inner, j.JoinType);

            Assert.AreEqual(0, j.Children.Count);

            Assert.IsInstanceOfType(typeof(Join), j1);
           
            Assert.AreEqual("DSC1", j1.BaseId);
            Assert.AreEqual("ID", j1.BaseColumn);

            Assert.AreEqual("DSC4", j1.JoineeId);
            Assert.AreEqual("ID", j1.JoineeColumn);

            Assert.AreEqual(Join.Type.Right, j1.JoinType);
            Assert.AreEqual(0, j1.Children.Count);
        }
    }
}
